
#ifndef QXMPPOMEMO_EXPORT_H
#define QXMPPOMEMO_EXPORT_H

#ifdef QXMPPOMEMO_STATIC_DEFINE
#  define QXMPPOMEMO_EXPORT
#  define QXMPPOMEMO_NO_EXPORT
#else
#  ifndef QXMPPOMEMO_EXPORT
#    ifdef QXmppOmemoQt6_EXPORTS
        /* We are building this library */
#      define QXMPPOMEMO_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define QXMPPOMEMO_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef QXMPPOMEMO_NO_EXPORT
#    define QXMPPOMEMO_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef QXMPPOMEMO_DEPRECATED
#  define QXMPPOMEMO_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef QXMPPOMEMO_DEPRECATED_EXPORT
#  define QXMPPOMEMO_DEPRECATED_EXPORT QXMPPOMEMO_EXPORT QXMPPOMEMO_DEPRECATED
#endif

#ifndef QXMPPOMEMO_DEPRECATED_NO_EXPORT
#  define QXMPPOMEMO_DEPRECATED_NO_EXPORT QXMPPOMEMO_NO_EXPORT QXMPPOMEMO_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef QXMPPOMEMO_NO_DEPRECATED
#    define QXMPPOMEMO_NO_DEPRECATED
#  endif
#endif

#endif /* QXMPPOMEMO_EXPORT_H */
