
#ifndef QXMPP_EXPORT_H
#define QXMPP_EXPORT_H

#ifdef QXMPP_STATIC_DEFINE
#  define QXMPP_EXPORT
#  define QXMPP_NO_EXPORT
#else
#  ifndef QXMPP_EXPORT
#    ifdef QXmppQt6_EXPORTS
        /* We are building this library */
#      define QXMPP_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define QXMPP_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef QXMPP_NO_EXPORT
#    define QXMPP_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef QXMPP_DEPRECATED
#  define QXMPP_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef QXMPP_DEPRECATED_EXPORT
#  define QXMPP_DEPRECATED_EXPORT QXMPP_EXPORT QXMPP_DEPRECATED
#endif

#ifndef QXMPP_DEPRECATED_NO_EXPORT
#  define QXMPP_DEPRECATED_NO_EXPORT QXMPP_NO_EXPORT QXMPP_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef QXMPP_NO_DEPRECATED
#    define QXMPP_NO_DEPRECATED
#  endif
#endif

#define QXMPP_BUILD_SHARED true
#define QXMPP_VERSION_MAJOR 1
#define QXMPP_VERSION_MINOR 16
#define QXMPP_VERSION_PATCH 1

#endif /* QXMPP_EXPORT_H */
